;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================

ORG 43AA			;17322

@Keyboard=02BB			;Keyboard scanning subroutine
@Decode=07BD			;Keyboard decode subroutine
@Laddr=09D8			;Line address subroutine
@DFile=400C			;Address of display file
@Spare1=4021			;16417 Spare 1, 1 Byte
@NxtLine=4029			;Next program line to be executed
@LastK=4025			;Keyboard debounce value
@MainForm=4089			;Stored mainform address

;================================
; Jump Table
;================================
	JP KeyLoop		;17322, Key loop and sorter
	JR LoadScr		;17325, Load main form
	JR SaveScr		;17327, Save main form
	JR Release		;17329, Test for keypress
	JP PrntPos		;17331,	Calculate Print Position
	JP File			;17334, File dropdown menu
	JP Scope		;17337, Scopescape dropdown menu
	JP Mumap		;17340, Scopescape dropdown menu
	JP Help			;17343, Help dropdown menu
	JP PopUps		;17346, Print Pop up message
@SPage
	00			;17349, Scopescape page number
@MPage
	00			;17352, Mumap page number
;================================
; Load screen
;================================
@LoadScr
	CALL Release
	PUSH HL
	LD HL,MainForm		;Set source
	LD DE,(DFile)		;Set destination
	JR +7
;================================
; Save screen
;================================
@SaveScr
	PUSH HL
	LD HL,(DFile)		;Set source
	LD DE,MainForm		;Set destination
	LD BC,0319		;BC=byte count
	LDIR			;Copy source to desination
	POP HL
	RET			;Return 0
;================================
; Wait for key release
;================================
@Release
	LD A,(LastK)		;Test for keypress
	CP FF			;Keypressed?
	JR NZ Release		;Yes, loop back
	RET
;================================
; Print dropdown menu
;================================
@DropDown
	CALL SaveScr		;Save main screen
	LD HL,(NxtLine)
@DropMenu
	LD BC,0005		;Skip REM header
	ADD HL,BC		;Start of data
	CALL Release		;Wait for key release
	LD B,(HL)		;Get Ycoord
	INC HL
	LD C,(HL)		;Get Xcoord
	INC HL			;Start of print data
	PUSH HL			;PUSH datapointer
	CALL PrntPos		;Get print position of the NW corner of dropdown
	POP BC			;POP Datapointer into BC
@DropY
	LD (Drop+1),HL		;Save start of row addr
@DropLoop
	LD A,(BC)		;Get chr to print
	CP 76			;EO REM?
	RET Z			;Return printing done
	CP 0C			;EOL marker ()?
	JR Z Drop		;Jump if EOL
	LD (HL),A		;Print A
	JR +7
@Drop
	LD HL,0000		;Get start of row addr
	ADD HL,DE		;Next row addr
	INC BC			;Increase data pointer
	JR DropY		;Start of row addr
	INC HL			;Increase print position
	INC BC			;Increase data pointer
	JR DropLoop
;================================
; File menu dropdown
;================================
@File
	CALL DropDown		;Print file dropdown menu
	CALL KeyScan		;Start Keyboard loop
	JR Z -5			;Loop until keypressed
	LD HL,FileKeys		;Valid keys table
	LD BC,000A 		;Byte count
	CPIR 			;Locate key
	JR NZ +1		;Jump if no match
	INC B			;B=Hi byte of line number
@SetLine
	LD C,(HL)
	LD H,B
	LD L,C
	CALL Laddr
	LD (NxtLine),HL
	JP Loadscr
;================================
@FileKeys			;Key code/lo byte of line number
	"S" 90			;400 Splash screen
	"M" 9A			;410 Manual
	"F" A4			;420 LOAD F-Scapes
	"P" AE			;430 LOAD Mandelplot
	"X" C2			;450 Exit
	0A			;EOTable marker=GOTO 10
;================================
; Scopescape dropdown
;================================
@Scope
	CALL DropDown		;Print Scopescape dropdown
	LD HL,0158 		;344, 501-157
	LD A,(SPage)		;Get current page number
	LD B,00
	LD C,A
	ADD HL,BC		;HL=501,502,503...
	CALL LAddr		;Get line address
	CALL DropMenu		;Print dropdown
	CALL KeyScan		;Start Keyboard loop
	JR Z -5			;Loop until keypressed
	CP "A"			;Key<"A"?
	JR NC +4		;Jump if key<"A"
;WrongKey
	CALL LoadScr		;Restore mainform
	RET			;Return to BASIC
	CP "W"			;Key>="W"?
	JR NC -8		;Jump if key>"V"
	SUB 25 			;A=0 to 22
	LD D,A 			;Save A			
	CALL Release		;Wait for key release
	LD A,D 			;Restore A
	CP 16			;A="V"
	JR NZ ScopePick		;Jump if not "V"
	LD HL,(DFile)		;Locate page number
	LD BC,030B
	ADD HL,BC
	LD A,(HL) 		;Get page number
	LD (SPage),A 		;Set page number
	JR Scope+3		;Print next page
@ScopePick
	LD A,(Spage) 		;Get page number
	SUB 9D			;A=0,1,2,3,4...
	AND A			;A=0?
	LD B,A			;Set DJNZ counter
	LD A,H			;A=0, Z flag not changed
	JR Z +4			;Jump if A was 0
	ADD A,15		;Add offset to A (A=A+21)
	DJNZ -4			;Loop until B=0
	ADD A,D			;Add item number to offset
	LD C,A 			;BC=Item number
	PUSH BC 		;PUSH Item number
	CALL LoadScr		;Restore screen
	POP BC 			;POP Item number
	RET			;Return Page number to BASIC
;================================
; Mumap dropdown
;================================
@Mumap
	CALL DropDown		;Print Mumap dropdown
	LD HL,016C 		;364, 521-157
	LD A,(MPage)		;Get current page number
	LD B,00
	LD C,A
	ADD HL,BC		;HL=521,522,523...
	CALL LAddr		;Get line address
	CALL DropMenu		;Print dropdown
	CALL KeyScan		;Start Keyboard loop
	JR Z -5			;Loop until keypressed
	CP "A"			;Key<"A"?
	JR NC +4		;Jump if key<"A"
;WrongKey
	CALL LoadScr		;Restore mainform
	RET			;Return to BASIC
	CP "W"			;Key>="W"?
	JR NC -8		;Jump if key>"V"
	SUB 25 			;A=0 to 22
	LD D,A 			;Save A			
	CALL Release		;Wait for key release
	LD A,D 			;Restore A
	CP 16			;A="V"
	JR NZ MuPick		;Jump if not "V"
	LD HL,(DFile)		;Locate page number
	LD BC,0313
	ADD HL,BC
	LD A,(HL) 		;Get page number
	LD (MPage),A 		;Set page number
	JR Mumap+3		;Print next page
@MuPick
	LD A,(Mpage) 		;Get page number
	JR ScopePick+3
;================================
; Help menu dropdown
;================================
@Help
	CALL DropDown		;Print help dropdown
	CALL KeyScan		;Start Keyboard loop
	JR Z -5			;Loop until keypressed
	LD HL,0262		;REM 610
@KeyH
	CP "H"			;"H"iRes keys
	JR NZ KeyM
	JR HelpPop
@KeyM
	CP "M"			;"M"enu Guide
	JR NZ KeyR
	LD L,6C			;REM 620
	JR HelpPop
@KeyR
	CP "R"			;"R"ead Me
	JR NZ KeyS
	LD L,76			;REM 630
	JR HelpPop
@KeyS
	CP "S"			;"S"capes Naming
	JR NZ KeyA
	LD L,80			;REM 640
	JR HelpPop
@KeyA
	CP "A"			;"A"bout
	LD L,8A			;REM 650
	JR Z HelpPop
@Goto10
	LD HL,+10		;GOTO 10
	JP SetLine+3
;===============================
; Help Popups
;================================
@HelpPop
	CALL LAddr
	LD (NxtLine),HL
	CALL LoadScr		;Reload main screen
	CALL DropDown+3		;Print REM 610, 620, 630 or 640
	CALL KeyScan		;Start Keyboard loop
	JR Z -5			;Loop until keypressed
	CALL LoadScr		;Reload main screen
	JR Goto10
;===============================
; Popups
;================================
@PopUps
	CALL DropDown		;Print popup 
	CALL KeyScan		;Start Keyboard loop
	JR Z -5			;Loop until keypressed
;Scape keys
	BIT 0,(IY+33)		;Which Popup keys to check
	JR NZ MuKeys		;If bit 0=1 then check Mu keys
	CP "B"
	JR Z PickOK
	CP "S"
	JR Z PickOK
	CP "A"
	JR Z PickOK
	JR NotOK
@MuKeys
	CP "M"			;"M"andelplot
	JR Z PickOK
	CP "D"			;"D"ithered
	JR Z PickOK
@NotOK
	XOR A
@PickOK
	PUSH AF			;PUSH Key code
	CALL LoadScr		;Reload main screen
	POP AF			;POP key code
	LD C,A			;Return key code to BASIC
	RET
;================================
; Print Position
;  B=Ycoord
;  C=Xcoord
;  BC unchange
;  On exit HL=Print Position address
;================================
@PrntPos
	PUSH BC			;PUSH YXcoords
	LD HL,0000		;Set accumulator to 0
	LD DE,0021		;Bytes/row
	LD A,B
	AND A			;Row=0?
	JR Z +3			;Yes, move on
	ADD HL,DE		;HL=33*row value
	DJNZ -3
	ADD HL,BC		;HL=33*row value+column offset
	LD BC,(DFile)		;Get DFile
	INC BC			;Skip 1st N/L chr
	ADD HL,BC		;HL=print position
	POP BC			;POP YXcoords
	RET	
;================================
; Test for keypress.
; On exit...
;   A=chr code
;================================
@KeyScan
	CALL Keyboard		;Call keyboard scanning routine
	LD B,H 			;BC=Raw Key values
	LD C,L 
	LD D,C			;Test for keypress, D=255 if no key press
	INC D 			;0?
	RET Z
	CALL Decode		;Call keyboard decode routine, BC to ZX81 codes
	INC A			;Set Z flag
	LD A,(HL)
	RET			;Return, A=chr code
;================================
; Key loop
;================================
@KeyLoop
	CALL KeyScan		;Get key press
	JR Z KeyLoop		;Loop until key pressed
	CP "F"			;"F"ile dropdown?
	JR NZ +4
	LD BC,+250
	RET			;GOTO 250
	CP "S"			;"S"copes dropdown?
	JR NZ +4
	LD BC,+490
	RET			;GOTO 490
	CP "M"			;"M"umaps dropdown?
	JR NZ +4
	LD BC,+495
	RET			;GOTO 495
	CP "H"			;"H"elp dropdown?
	JR NZ +4
	LD BC,+600
	RET			;GOTO 600
	CP "X"			;E"x"it key?
	JR NZ +4
	LD BC,+700
	RET  			;GOTO 700
	CALL Release
	JR KeyLoop