;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================

ORG 4084

@Keyboard=02BB			;Keyboard scanning subroutine
@Decode=07BD			;Keyboard decode subroutine
@LastK=4025			;Key press value, 16421
@WRX=4084
@HRDFile=2000			;Hires DFile
@LineStor=3800			;Temp line store for flips

;================================
; WRX Video driver by Wilf Rigter
; WRX16K - 1996
; Annotations by Wilf Rigter
;================================
@WRX
	LD IX,HR		;Simple start of the hres mode
	RET
@LBuf
	LD R,A			;Load HFILE address LSB
	80 80 80 80		;32 NOPs = 256 pixels
	80 80 80 80 		;80=Black on white display
	80 80 80 80 		;00=White on black display
	80 80 80 80 
	80 80 80 80 
	80 80 80 80 
	80 80 80 80 
	80 80 80 80 
	RET NZ			;Always returns
@HR    
	LD B,06			;Delay
@HR0
	DJNZ HR0
	NOP			;More delay fine tuning
	NOP
	NOP
	NOP
	NOP
	DEC B
@HDAddr
	LD HL,HRDFile		;HRDFile
	LD DE,20		;32 bytes per line
	LD B,C0			;192 lines per hires screen
@HR1
	LD A,H			;Get HFILE address MSB
	LD I,A			;Load MSB into I register
	LD A,L			;Get HFILE address LSB
	CALL C089		;CALL Lbuf+8000
	ADD HL,DE		;Next line
	DEC B			;Dec line counter
	JP NZ HR1		;Last line
@HR2   
	CALL 0292		;Return to application program
	CALL 0220		;Extra register PUSH and VSYNC
;================================
;Break key test. This code segment is optional
	CALL 0F46		;Check break key
	LD A,1E			;Restore pattern table pointer
	LD I,A
	JR NC Stop		;Skip the HR vector load if BREAK
;================================
	LD IX,HR		;Load the HR vector
@Stop
	JP 02A4			;Return to application program
;================================
;Jump Table
	JR Display		;16608
	JR LoRes		;16610
;================================
; Delay loop
;================================
@Delay
	LD BC,0001 		;POKE xxxxx from BASIC
@DLup
	DEC BC 
	LD A,B 
	OR C 
	JR NZ DLup
	RET
;================================
@Display
	CALL WRX		;Display image
@KeyLoop			;Sort out the key presses
	CALL Keyscan		;Page keyboard
	CP FF
	JR Z KeyLoop		;Stay in loop until key pressed
	Ld B,00
	LD C,A			;Save key code to C
	CALL Wait		;Wait for key release
	LD A,C			;Restore keycode to A, BC=Keycode
@KeyI
	CP "I"			;Invert?
	JR NZ KeyP		;No, move on
	CALL Invert		;Yes, invert screen
	JR KeyLoop
@KeyP
	CP "P"			;Palette cycle?
	JR NZ KeyV		;No, move on
@Cycle
	CALL Invert		;Invert display
	CALL Delay		;Cycle delay
	CALL 02BB		;Check for key press
	INC L 
	JR Z Cycle		;Loop until key pressed
	JR KeyLoop		;Back to wait mode
@KeyV
	CP "V"			;Verticle flip
	JR NZ KeyH		;No, move on
	CALL VFlip
	JR KeyLoop
@KeyH
	CP "H"			;Horizontal flip
	JR NZ KeyC		;No, move on
	CALL HFlip
	JR KeyLoop
@KeyC
	CP "C"			;Delay value
	JR Z LoRes
@KeyR
	CP "R"			;EmuMenu
	JR Z LoRes
@KeyQ
	CP "Q"			;Quit?
	JR NZ KeyLoop
;================================
@LoRes
	PUSH BC			;PUSH Keycode
	CALL 0F2B		;Slow
	LD B,(IY+82) 
@SyncLup
	LD A,(4034)		;Get frames
	CP B 
	JR Z SyncLup
	LD A,1E 
	LD I,A 
	LD IX,0281 
	POP BC			;POP keycode return C,R, or Q to BASIC
	RET
;================================
; Test for keypress.
; On exit...
;  A=key chr code
;================================
@KeyScan
	CALL Keyboard		;Call keyboard scanning routine
	LD B,H 			;BC=Raw Key values
	LD C,L 
	LD D,C			;Test for keypress , D=FF if no key
	LD A,D			;A=FF if no key press
	INC D 			;D=0 if no key
	RET Z 			;No keypress
	CALL Decode		;Decode routine converts BC to ZX81 keycodes
	LD A,(HL)		;A=CHR CODE
	RET
;================================
; Wait for key release
;================================
@Wait
	LD A,(LastK)		;Get keyboard status
	CP FF			;Key pressed?
	JR NZ Wait		;Jump if keypress
	RET
;================================
; Invert the WRX buffer
;================================
@Invert
	LD HL,408B 
	LD BC,0020 
	LD A,(HL) 
	ADD A,80 
	LD (HL),A 
	DEC BC 
	LD D,H 
	LD E,L 
	INC DE 
	LDIR 
	RET
;================================
; WRX Vertical flip
;================================
@VFlip
	LD HL,HRDFile		;Start of HRDFile
	LD (LineStor),HL	;Save top row addr
	LD BC,17E0		;Offset to bottom row
	ADD HL,BC		;HL=Address of bottom row.
	LD B,60			;Rows to process counter
	LD A,20			;Constant
@VLoop
	PUSH BC			;PUSH counter
	LD (LineStor+2),HL	;Save bottom row addr
	LD HL,(LineStor)	;Transfer top row data to storage area
	LD DE,LineStor+4
	LD BC,0020
	LDIR
	LD HL,(LineStor)	;Transfer bottom row data to top row address
	EX DE,HL
	LD HL,(LineStor+2)
	LD C,A
	LDIR
	LD (LineStor),DE	;Save top row address
	LD HL,LineStor+4	;Transfer top row data to bottom row address
	LD DE,(LineStor+2)
	LD C,A
	LDIR
	LD C,A			;Calculate address of next bottom row
	LD HL,(LineStor+2)
	AND A			;Clear CY
	SBC HL,BC
	POP BC			;POP row counter
	DJNZ VLoop		;Repeat if NZ
	LD C,B			;Return 0 to BASIC
	RET
;================================
; WRX Horizontal flip
;================================
@HFlip
	LD HL,HRDFile		;HL=Left column
	LD DE,201F
	LD B,C0			;Row counter 
@Outer
	PUSH BC			;PUSH row counter
	LD B,10			;Column counter
@Inner
	PUSH BC			;PUSH column counter
	LD A,(HL) 		;Get left column value
	LD BC,0008
@Left
	RLA			;Flip left value bit pattern
	RR B
	DEC C
	JR NZ Left
	LD A,(DE)		;Get right column byte
	EX DE,HL		;HL=right, DE=left
	LD (HL),B 		;Copy flipped left value right
	LD BC,0008
@Rite
	RLA			;Flip right value bit pattern
	RR B
	DEC C
	JR NZ Rite
	EX DE,HL		;HL=left, DE=right
	LD (HL),B 		;Copy right flipped value left
	INC HL 			;Left+1
	DEC DE 			;Right-1
	POP BC			;POP column counter
	DJNZ Inner		;Repeat x 16 columns
	LD C,10 		;Start next row
	ADD HL,BC		;Next left row
	EX DE,HL		;DE=left
	LD H,D
	LD L,E
	LD C,1F
	ADD HL,BC		;Next right row
	EX DE,HL		;HL=left, DE=right
	POP BC			;POP row counter
	DJNZ Outer		;Repeat x 192 rows
	LD C,B			;Return 0 to BASIC
	RET  
;================================